#!/usr/local/bin/perl -w
#
# $Id: getdata.pl,v 1.3 2002/02/07 11:59:53 dismented Exp $
#

# getdata.pl: Fetches IP-Accounting-Tabels from a Cisco Router via SNMP
# getdata.pl is only for fetching the data (recommended e.g. all 2-3 Minutes)

# Version 1.0: First Use
# Version 1.1: - Bugfixes (Directories are made when needed)
#	       - The "Network is unreachable" - Case is written down in the Logfile
#	       - nicer Log-file-Messages

# Tobias Geiger
#
# Version 1.3: Several Great Improvments by Anatoly Ivanov, please read the readme-file
# Version 1.3.1: Several bugfixes (%new_traffic bug, DOS EOL bug)
# Version 1.4: all changes are in summarize.pl

use Net::SNMP 4.0;
use Getopt::Std;    # For getopts()
use File::Basename; # For basename($0) and dirname($0)
	
$ck='1.3.6.1.4.1.9.2.4.6.0';
$ac='1.3.6.1.4.1.9.2.4.9.1.4';
$cp='1.3.6.1.4.1.9.2.4.11.0';

my $dirname=dirname($0);
my $config_file="$dirname/getdata.conf";	# Default-Configfile: getdata.conf in the same Directory as getdata.pl
my $progname=basename($0);		  	# Programmname
my $helptext=
"\n$progname v1.4 by Anatoly Ivanov, Tobias Geiger\n".
"\nUsage:\n".
"\t$progname [-c config-file]\n".
"\t  -c <config-file> - points to the config file to use; overridess the default config file: $config_file\n";

my (@routername,@ip,@community,$logpath,$getdata_errorlogfile,$summarize_errorlogfile,$lostbytesfile,%new_traffic); 
my $routeranzahl=0;		# increments by 1 for each configured router
my $cfgzeilennummer=0;		# to write down the linenumber in the config-file, in case of config-file-parse-errors
use vars qw/$opt_h $opt_c/;	# don't know exactly why this is needed, but it's used by Getopt
my $debug=0;			# default value

# parse Command-Line:
die $helptext unless getopts('hc:');
if (defined($opt_h)) { print $helptext; exit(0);}
if (defined($opt_c)) { $config_file="$opt_c";}

# parse Config-file
readconfigfile($config_file);


# for every configured router 
for ($x = 1 ; $x <= $routeranzahl ; $x++) 

    {
	my ($session, $e) = Net::SNMP->session(
		-hostname  => $ip[$x],
		-community => $community[$x]);
	die $e unless defined($session);
	# do the 3 subs with apropriate parameters
	#
	undef %new_traffic;
	check_router($session);	
        get_new_traffic($session); 
	save_new_traffic($routername[$x]); 
     }                                                                                            

exit(0);


sub readconfigfile {                           

my $config_file=$_[0];                                                                                        
open (CONF, "<$config_file") || die "couldn't open configfile $config_file\n";
                                                                                                                                       
while (<CONF>) {                                                                                                                       
                                                                                                           
    chomp $_;                                                                                 
    s/\r//g; # remove the DOS EOL symbols
    
    $cfgzeilennummer++;                                  
                        
    if ( substr($_,0,1) eq "[" ) {                                                                                                     
        $routeranzahl++;                                                                                                               
        my $routernamenlaenge=index($_,"]") ;                                                                                          
        if ($routernamenlaenge == -1) { die "wrong config-entry in $config_file, line $cfgzeilennummer\n"};
	$routername[$routeranzahl]=substr($_,1,$routernamenlaenge-1);                                                                  
        next;                                                                                                                          
    }                                                                                                                                  
                                                                                                                   
    elsif (m/^ip\s*=\s*(.*)/i ) {$ip[$routeranzahl]=substr($_,3); next;}   
    elsif (m/^read.write.community\s*=\s*(.*)/i ) {$community[$routeranzahl]=substr($_,21); next;}   
    elsif (m/^outputpath\s*=\s*(.*)/i) {$logpath=substr($_,11); next;}  
    elsif (m/^getdata errorlogfile\s*=\s*(.*)/i) {$getdata_errorlogfile=substr($_,21); next;} 
    elsif (m/^summarize.errorlogfile\s*=\s*(.*)/i) {$summarize_errorlogfile=substr($_,23); next;}
    elsif (m/^lostbytesfile\s*=\s*(.*)/i) {$lostbytesfile=substr($_,14); next;}
    elsif (m/^debug\s*=\s*(.*)/i) { $debug=($1=~m/^yes/i)?1:0; next;}
}                                                                                                                                      
close(CONF);                                                                                                                           
die "logpath not defined" unless(defined $logpath);
if ($routeranzahl==0) {die "no routers definied (perhaps due to a Syntax Error in $config_file)\n";}
} #subEnd   

sub check_router {
	
	my $s=$_[0];
	
	# check for lost-bytes and log them	
	die ($s->error()) unless (defined($res=$s->get_request(-varbindlist => [$ck])));
	if ($res->{$ck} > 0) {
		my ($sec,$min,$hour,$mday,$month,$year) = localtime;                                                     
		my $timestamp=sprintf("%d/%02d/%02d-%02d:%02d:%02d", $year+1900,++$month,$mday,$hour,$min,$sec);     
		my $realpath="$logpath\/$_[0]";                                                                   
		if ( ! -d $realpath) { mkdir ("$realpath",0777);}                                                        
		my $realfile="$realpath\/$lostbytesfile";  
		open (LBFILE, ">>$realfile") || die "couldn't open the LOSTBYTES-Logfile $realfile\n";       
		print LBFILE "$timestamp: $res->{$ck}\n"; 
		close(LBFILE);     
	}
	
	# get and set checkpoint: necessary to read the accounting data
	die $s->error() unless (defined($res=$s->get_request(-varbindlist => [$cp])));
	unless ($debug){
		# write the checkpoint only in production environment
		die $s->error() unless (defined($s->set_request(-varbindlist =>[$cp,INTEGER32,$res->{$cp}])));
	} else {
		print "Warning: running in debug mode!\n";
	}
} #subEnd

sub get_new_traffic {
	
	my $s=$_[0];
	my $traffic_entry;
	my @args=(-varbindlist => [$ac]);
	while (defined($resp=$s->get_next_request(@args))) 
	{
		$base = (keys(%{$s->var_bind_list}))[0];
		last unless (Net::SNMP::oid_base_match($ac,$base));
		if ($base =~ m/^$ac\.(\d+\.\d+\.\d+\.\d+)\.(\d+\.\d+\.\d+\.\d+)/)
		{
			$new_traffic{$1}{$2}+=$resp->{$base};
		} 
		else 
		{
			error_log("wrong snmp-data: $base->",$resp->{$base});
		}
		@args = (-varbindlist => [$base]);
	}
} #subEnd


sub save_new_traffic {

	my ($sec,$min,$hour,$mday,$month,$year) = localtime;                                                 
	my $filename=sprintf("%02d-%02d-%d.raw", $mday,++$month,$year+1900);
	my $realpath="$logpath\/$_[0]";                                                                   
	if ( ! -d $realpath) { mkdir ("$realpath",0777);}                                                        
	my $newsafename = "$logpath\/$_[0]\/$filename";
	open (NEWSAVEFILE,">>$newsafename") || error_log("couldn't make new file: $newsafename",$_[0]);
	my ($src, $dst);
	foreach $src (sort keys %new_traffic) {
	    foreach $dst (sort keys %{$new_traffic{$src}}) {
		print NEWSAVEFILE "$src $dst $new_traffic{$src}{$dst}\n";
	    }
	}
	close(NEWSAVEFILE);
} #subEND

sub error_log {

my $errormsg = "$_[0]";
my $errorrouter = "$_[1]";
my ($sec,$min,$hour,$mday,$month,$year) = localtime;                                                 
my $timestamp=sprintf("%d/%02d/%02d-%02d:%02d:%02d", $year+1900,++$month,$mday,$hour,$min,$sec);
my $realpath="$logpath\/$errorrouter";
if ( ! -d $realpath) { mkdir ("$realpath",0777);}
my $realerrorfile="$realpath\/$getdata_errorlogfile";
open (ERRORFILE, ">>$realerrorfile") || die "couldn't open Error-Logfile $realerrorfile\n";
print ERRORFILE "$timestamp: $errormsg\n";
close(ERRORFILE);

} #subEND

