#!/usr/bin/perl -w
#
# $Id: summarize.pl,v 1.5 2002/02/07 11:59:53 dismented Exp $
#

# summarize.pl

# v1.0: First use
# v1.1: Bugfixes: -c Option, as in getdata.pl 
# v1.3: Several Great Improvments by Anatoly Ivanov, please read the readme-file
# v1.4: 
# 	- some improvements on summarizing by subnets (thnx to Denis Tokarev)
# 	- added min_traffic parameter
# 	- ignore parameter is deprecated, use ignore_ip instead
# 	- ignore_ip is now also in Net::IP format

use Getopt::Std;    # For getopts()   
use File::Basename; # For basename($0) und dirname($0) 
use Socket;
use Net::IP;

my (@routername,$logpath,$summarize_errorlogfile,$lostbytesfile);
my $dirname=dirname($0);                                                                                                                      
my $config_file="$dirname/getdata.conf";        # Default-Configfile: getdata.conf 
						# in the same directory as summarize.pl 
my $progname=basename($0);                      # Programmname                                                                                
my $helptext=
"\n$progname [-c config-file] \| [-h] \| ([-D Day] \| [-M Month] \| [-d] \| [-m]) [-X]\n\n".
"-c: config file to use (overrides default: $config_file)\n".
"-d: summarize daily traffic yesterday (or today with -U)\n".
"-m: summarize last month (or current with -U)\n".
"-U: forces to summarize today/this month info\n".
"-D Day: summarize specified Day (Format: 2 Decimals <= 31)\n".
"-M Month: summarize specified Month (Format: 2 Decimals <= 12)\n".
"-H Generate HTML output\n\n".
"-X: Delete Temporary files (temp.dat and *.raw) after the summarize-prozess\n\n\n".
"$progname v1.4 by Anatoly Ivanov, Tobias Geiger\n\n";

# parse command-line
use vars qw/$opt_h $opt_c $opt_D $opt_M $opt_d $opt_m $opt_X $opt_H $opt_U/;
die $helptext unless getopts('c:hD:M:dmHUX');

my $routeranzahl=0;    
my $cfgzeilennummer=0;
my ($xxsec,$xxmin,$xxhour,$gesterntag,$gesternmonat,$gesternjahr) 
	= localtime(time()-((defined($opt_U))?30:86400));
#
# yesterday or 30 seconds ago :)
# 30 seconds is needed for cron coincidances :)
#
$gesternjahr	= $gesternjahr+1900;
$gesterntag	= sprintf("%02d",$gesterntag);
$gesternmonat	= sprintf("%02d",++$gesternmonat);
print "Accounting on:$gesternjahr-$gesterntag-$gesternmonat\n" if ($debug);
#print "Accounting on:$gesternjahr-$gesterntag-$gesternmonat\n";
if (defined($opt_d) || defined($opt_D) ){
 $AccOn = "Day: $gesternjahr-$gesterntag-$gesternmonat";
}
if (defined($opt_m) || defined($opt_M) ){
 $AccOn = "Month: $gesternjahr-$gesternmonat";
}
#$gesternmonat="05"; #DEBUG-TEST fr -m

my $html_color1="#69a9c8";
my $html_color2="#BDDBED";
my $html_color3="#a1cfe6";

my $subnet_num = -1;

# read configfile                          
if (defined($opt_c)) { $config_file="$opt_c";} 
readconfigfile($config_file);

if (defined($opt_h)) { print $helptext; exit(0);}                                           
if (defined($opt_H)) { $HTML=1; }
if (defined($opt_d)) 
{ 
	lastday(); 
	if (defined($opt_m))
	{
		lastmonth();
	}
	exit(0); 
}   
if (defined($opt_m)) 
{ 
	lastmonth();
	exit(0); 
}
if (defined($opt_D)) { 
			if ( $opt_D > 31 ) { die "Parameter isn't a valid day\n";}
			$gesterntag=sprintf("%02d",$opt_D);
			lastday();
			exit(0);
		     }
if (defined($opt_M)) {
			if ( $opt_M >12 ) { die "Parameter isn't a valid month\n";}
			$gesternmonat=sprintf("%02d",$opt_M);	     
			lastmonth();
			exit(0);
		    }
# we only get to this point, when no command-line parameter is defined
die $helptext;   

#############################
# reading the config, setting the defaults
#
# expected vars:
#   original:
# 	$outpath
# 	$summarize errorlogfile
# 	$lostbytesfile
#
#   added by avi:
#	$debug=0
# 	$html_outpath
# 	
# 	$units=K //kilobytes
# 	$signs=2 // two signs after decimal point
#
# 	$ignore="" // source of traffic to ignore
#	$group_by=0 //destination

sub readconfigfile { 

my $config_file=$_[0];
my $ignore_index=0;
my $subnet_index=0;

##########
# init the default values
#
$units="K";
$signs=2;
$grp_by=0;
$group_by="dst";
$limit=10; # number of detailed conversations 
$min_traffic=0;

open (CONF, "<$config_file") || die "couldn't open configfile $config_file\n";
while (<CONF>) {
    chomp $_;
    s/\r//; #remove DOS EOL
    $cfgzeilennummer++;                                                                                                                       
    next if (m/^#/); #comment 

        if ( substr($_,0,1) eq "[" ) 
	{   
                $routeranzahl++;                                                                                                                      
                my $routernamenlaenge=index($_,"]") ;                                                                                                 
                if ($routernamenlaenge == -1) 
		{ 
			die "wrong configfile-entry in $config_file, line $cfgzeilennummer\n"
		};                            
                $routername[$routeranzahl]=substr($_,1,$routernamenlaenge-1);                                                                         
                next;                                                                                                                                 
        }                         

	elsif (m/^outputpath\s*=\s*(.*)/i) {$logpath=$1; next;} 
	elsif (m/^html.+path\s*=\s*(.*)/i) {$html_outpath=$1;  next;} 
	elsif (m/^css.url\s*=\s*(.*)/i) {$css_url=$1;  next;} 
	elsif (m/^summarize.+errorlogfile\s*=\s*(.*)/i) {$summarize_errorlogfile=$1; next;} 
        elsif (m/^lostbytesfile\s*=\s*(.*)/i) {$lostbytesfile=$1; next}             
	elsif (m/^ignore\s*=\s*(.*)/i) { 
		die "\"ignore\" parameter is deprecated, use ignore_ip instead";
	}
	elsif (m/^ignore.+ip\s*=\s*(.*)/i) { 
		$ignore_ip[$ignore_index++]=
			new Net::IP ($1) || die "Invalid ignore subnet"; 
		next;
	} 
	elsif (m/^subnet_sum\s*=\s*(.*)/i) { 
		$subnet_sum[$subnet_index]=$1; 
	 	$subnets[$subnet_index] = 
			new Net::IP ($subnet_sum[$subnet_index]) || die "Invalid sum_subnet";
	 	$subnet_index++;
         	print "subnet_sum = $subnet_sum[$subnet_index-1]\n" if ($debug);
		next;
        } 
	elsif (m/^debug\s*=\s*(.*)/i) { $debug=($1=~m/^yes/i)?1:0; next;}
	elsif (m/^resolve\s*=\s*(.*)/i) { $resolve=($1=~m/^yes/i)?1:0; next;}
	elsif (m/^signs\s*=\s*(\d+)/i) { $signs=$1; next;}
	elsif (m/^min.+traffic\s*=\s*(\d+)/i) { $min_traffic=$1; next;}
	elsif (m/^limit\s*=\s*(\d+)/i) { $limit=$1; next;}
	elsif (m/^group.*by\s*=\s*(\d+)/i) 
	{ 
		$group_by=$1; 
		if ($group_by=~/^dst/i){
			$grp_by=0;
		} 
		elsif ($group_by=~/^src/i)
		{
			$grp_by=1;
		}
		else
		{
			die "group_by value is invalid: $group_by\n";
		}
		next;
	}
	elsif (m/^units\s*=\s*(.*)/i) 
	{ 
		$units=$1; 
		unless ($units=~m/g|m|k|b/i){
			die "wrong units: $units; only G,M,K and b are accepted";
		}
		next;
	} 

}
                                                                                                                                              
close(CONF);

$subnet_num = $subnet_index;

if ($units =~ m/g/i) { $divisor=1024*1024*1024; }
elsif ($units =~ m/m/i) { $divisor=1024*1024; }
elsif ($units =~ m/k/i) { $divisor=1024; }
else { $divisor=1; };
	
die "logpath not defined" unless(defined $logpath);
die "html_path not defined" if(!defined($html_outpath) && $HTML);

if ($routeranzahl==0) {die "no routers defined (Configfile Syntax Error in $config_file)\n";}                                                  
} #subEnd



sub get_sum_and_save_traffic 
{



# here redundand ip-combinations are summarized an saved in a new file: $DAY.raw
    
    my (%summed_traffic,%sum_traffic, %dest_traffic, %raw_traffic);
    my (%summed_subnet);

#first read the saved file:

     my $loadfile="$_[0]";
     my $linenumber=0;
     open (LOADDATASET, "<$loadfile") || error_log ("couldn't open $loadfile: $!",$_[2]);
         while (<LOADDATASET>) {
             $linenumber++;
             $_ =~ /(\d+\.\d+\.\d+\.\d+)\s(\d+\.\d+\.\d+\.\d+)\s(\d+)/ 
			|| error_log ("Wrong Data in $loadfile, line $linenumber: $_",$_[2]);
             $raw_traffic{$1}{$2} += $3;
         }
     close (LOADDATASET);

# then summarize

    my ($raw_src, $raw_dst);

   
    foreach $raw_src (sort keys %raw_traffic) 
    {
	if (ignore($raw_src)){
		print "Ignoring traffic for $raw_src\n" if ($debug);
		next;
	}
	foreach $raw_dst (sort keys %{$raw_traffic{$raw_src}}) 
           {

	        my $i = -1;
	        $i = in_subnet($raw_dst);
		if ( $i >= 0 ) {
		   $summed_subnet{$i} += $raw_traffic{$raw_src}{$raw_dst};	
		   #print "summed_subnet = $summed_subnet{$subnet_index}";
		}
	
		$summed_traffic{$raw_src}{$raw_dst} += $raw_traffic{$raw_src}{$raw_dst};
		if ($HTML) 
		{
			# we keep this for HTML output
			if ($grp_by==0)
			{
				$dest_traffic{$raw_dst} += $raw_traffic{$raw_src}{$raw_dst};
				$sum_traffic{$raw_dst}{$raw_src} += $raw_traffic{$raw_src}{$raw_dst};
			} 
			else
			{
				$dest_traffic{$raw_src} += $raw_traffic{$raw_src}{$raw_dst};
				# yes, in this case this is the same as $summed_traffic :)
				$sum_traffic{$raw_src}{$raw_dst} += $raw_traffic{$raw_src}{$raw_dst};
			}

		}
		# if ( exists $summed_traffic{"$raw_dst"}{"$raw_src"}) 
		#{
		#$summed_traffic{$raw_dst}{$raw_src} += $raw_traffic{$raw_src}{$raw_dst};
		#}
        	#else  {
		#$summed_traffic{$raw_src}{$raw_dst} += $raw_traffic{$raw_src}{$raw_dst};
		#}
           }
    }

# and then save

my $safefile=$_[1];
my ($sum_src, $sum_dst);                                                                                         
open (SAVEFILE,">$savefile") || error_log("couldn't make $safefile",$_[2]);
    foreach $sum_src (sort keys %summed_traffic) 
        {
		foreach $sum_dst (sort keys %{$summed_traffic{$sum_src}})
	       	{
			printf SAVEFILE "%s %s %.0f\n",$sum_src,$sum_dst,$summed_traffic{$sum_src}{$sum_dst}; 
		}
	}
close(SAVEFILE);

# and generate the HTML output
  if ($HTML)
  { 
        my $HTMLfile="$_[3]";
	my $iaddr;
	my $resolved;
		
	open (HTML,">$HTMLfile") || error_log("cannot create $HTMLfile: $!");
	print HTML <<HTML;
<html>
<head>
	<title>IP accounting for $_[2] (grouped by $group_by)</title>
HTML
if (defined($css_url))
{
	print HTML "<link rel=stylesheet type=\"text/css\" href=\"$css_url\">";
}
my $lastupdate=localtime(time);
print HTML <<HTML;
</head>
<body>
<h2>Last-update: $lastupdate</h2>

<h3>Accounting on [$AccOn]</h3>

<a name="home">
<table width=500>
<tr><td>
HTML
#
# if we have subnetting summarization...
# 
if ($subnet_num>0) {
print HTML<<HTML;
<table width=100% border=1>
	<tr>
		<td class=tableCaption>Subnet</td>
		<td class=tableCaption>Traffic ($units)</td>
	</tr>

HTML
	for ( $i=0; $i<$subnet_num; $i++ ) {
	 print HTML <<HTML; 
         <tr><td class=tableCell>$subnet_sum[$i]</td><td class=tableCell>
HTML
	 $summed_subnet{$i}=0 unless defined $summed_subnet{$i};
	 printf HTML "%.${signs}f", $summed_subnet{$i}/$divisor;
	 printf HTML "</td></tr>";
	} 	

print HTML <<HTML;
</table>
<hr>
HTML

} #endif $subnet_num>0

print HTML<<HTML;
<table width=100% border=1>
	<tr>
		<td class=tableCaption>IP address</td>
		<td class=tableCaption>Traffic ($units)</td>
	</tr>
HTML


	$total=0;
	foreach $dest(sort {$dest_traffic{$b} <=> $dest_traffic{$a}} keys(%dest_traffic))
	{
	  if ($dest_traffic{$dest}>=$min_traffic)
	  {
		print HTML<<HTML;
	<tr>
	<td class=tableCell><a href="#$dest">${dest}</a></td>
	<td class=tableCell>
HTML
		printf HTML "%.${signs}f", $dest_traffic{$dest}/$divisor;
		print HTML "</td></tr>";
	  }
		$total+=$dest_traffic{$dest};
	}
	  
	printf HTML "<tr class=tableLastCell><td>total</td><td>%.${signs}f</td></tr>",$total/$divisor;
	print HTML "</table>";
    	foreach $sum_dst (sort keys %sum_traffic) 
        {
	  if ($dest_traffic{$sum_dst}>=$min_traffic)
	  {
		if ($resolve)
		{
			$iaddr=inet_aton($sum_dst);
			$resolved=gethostbyaddr($iaddr, AF_INET);
		}
		$resolved="n/a" unless(defined($resolved));
		print HTML <<HTML;
<hr>
<a name=\"$sum_dst\">
Go to <a href="#home">summary</a>
<table width=100% border=1>
	<td class=secondCaption>Top $limit for</td><td class=secondCaption>$sum_dst<br>$resolved</td>
</tr><tr>
	<td class=secondTitle>IP:</td><td class=secondTitle>Traffic ($units)</td>
</tr>

HTML
		$i=0; 
		foreach $sum_src (sort { $sum_traffic{$sum_dst}{$b} <=> $sum_traffic{$sum_dst}{$a} } keys %{$sum_traffic{$sum_dst}})
	       	{
			printf HTML "<tr><td class=secondCell>$sum_src</td><td class=secondCell>%.${signs}f</td>",$sum_traffic{$sum_dst}{$sum_src}/$divisor; 
			printf HTML "</tr>";
			last unless ($i++<$limit);
		}
		print HTML "</table>";
	  } #endif $dest_traffic{$sum_dst}>=$min_traffic
	}

print HTML "</td></tr></table></body></html>";

  } #end if ($HTML)
} #endSub


sub lastday {	# Summarize the last Day: 
	
	for ($x = 1 ; $x <= $routeranzahl ; $x++)  
	{
		$loadfile="$logpath\/$routername[$x]\/$gesterntag-$gesternmonat-$gesternjahr.raw";
		$savefile="$logpath\/$routername[$x]\/$gesterntag-$gesternmonat-$gesternjahr.log";
		$htmlfile="$html_outpath/$routername[$x]/$gesternjahr/$gesternmonat/$gesterntag.html";
		mkdir_p("$html_outpath","$routername[$x]/$gesternjahr/$gesternmonat");

		get_sum_and_save_traffic($loadfile,$savefile,$routername[$x],$htmlfile);
		if (defined($opt_X))
		{ 
			if ( ! unlink("$logpath\/$routername[$x]\/$gesterntag-$gesternmonat-$gesternjahr.raw") == 1)
			{
				error_log("couldn't delete $logpath\/$routername[$x]\/$gesterntag-$gesternmonat-$gesternjahr.raw",$routername[$x]);
			}
		}
	}

} #endSub



sub lastmonth { # Summarize the last month
    for ($x = 1 ; $x <= $routeranzahl ; $x++)
    {
	# determine year-directory and make it if needed
	my $jahresverz="$logpath\/$routername[$x]\/$gesternjahr";
	mkdir_p ($logpath,"$routername[$x]\/$gesternjahr");
	# make a Pattern matching only relevant .log-files and fill it
	my $glob_pattern="$logpath\/$routername[$x]\/??-$gesternmonat-$gesternjahr.log";
	my @dotlogfiles = glob($glob_pattern);
	
	# make one big file out of all these .log-files (dirty, but so i can use the get_sum_and_save function ;)
	open (EINEFUERALLE,">$logpath\/$routername[$x]\/temp.dat");
	foreach $dotlogfile (@dotlogfiles) 
	{
	    print "processing file: $dotlogfile\n" if ($debug);
	    open (EINES,"<$dotlogfile");
	while (<EINES>) { print EINEFUERALLE $_; }
	    close (EINES);
	    # if -X option, then delete .log-file 
	    if (defined($opt_X)){ if ( ! unlink("$dotlogfile") == 1)
	    {
		    error_log("couldn't delete $dotlogfile",$routername[$x]);
	    }
    	}
   }
   close (EINEFUERALLE);
	
        
   # now use the get_sum_and_save function for this temp.dat
   $loadfile="$logpath\/$routername[$x]\/temp.dat";               
   $htmlfile="$html_outpath\/$routername[$x]\/$gesternjahr/$gesternmonat.html";               
   mkdir_p ("$html_outpath","$routername[$x]\/$gesternjahr",0777);
   $savefile="$jahresverz\/$gesternmonat-summed.log";               
   get_sum_and_save_traffic($loadfile,$savefile,$routername[$x],$htmlfile);  
   if (defined($opt_X))
   {
	   if ( ! unlink("$logpath\/$routername[$x]\/temp.dat") == 1)
	   {
		   error_log("couldn't delete $logpath\/$routername[$x]\/temp.dat",$routername[$x])
	   }
   }	
    
    }
} #endSub


sub error_log {

my $errormsg = "$_[0]";                                                                                                                       
my $errorrouter = "$_[1]";                                                                                                                    
my ($sec,$min,$hour,$mday,$month,$year) = localtime;                                                                                          
my $timestamp=sprintf("%d/%02d/%02d-%02d:%02d:%02d", $year+1900,++$month,$mday,$hour,$min,$sec);                                              
my $realpath="$logpath\/$errorrouter";                                                                                                        
if ( ! -d $realpath) { mkdir ("$realpath",0777);}                                                                                             
my $realerrorfile="$realpath\/$summarize_errorlogfile";                                                                                         
open (ERRORFILE, ">>$realerrorfile") || die "couldn't make $realerrorfile\n";
print ERRORFILE "$timestamp: $errormsg\n";                                                                                                    
close(ERRORFILE);  

} #subEnd

sub ignore {
	my $src = new Net::IP ($_[0]);
	my $mask;
	foreach $mask (@ignore_ip){
		return 1 if ( $mask->overlaps($src) == -2) ; 
	}
	return 0;
}

sub in_subnet {
	my $i;
	my $dst=$_[0];
	my $ip = new Net::IP ($dst);

	for ( $i = 0; $i < $subnet_num; $i++ ) {
	 if ( ($subnets[$i]->overlaps($ip) == -2 )) {
		return $i;

	 }
	}
  	return -1;
}


sub mkdir_p {
	my $permanent=$_[0];
	my $var=$_[1];
	unless ( -d $permanent )
	{
		die "$permanent drectory doesn't exists!";
	}
	@dir=split(/\//,$var);
	foreach $var (@dir)
	{
		$permanent="$permanent/$var";
		unless ( -d $permanent )
		{
			mkdir ($permanent,0777) || die "Cannot create directory $permanent: $!";
		}
	}
}
		
